package com.yizhi.forum.application.controller.remote;


import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.Posts;
import com.yizhi.forum.application.feign.remote.RemotePostsClient;
import com.yizhi.forum.application.service.IPostsAppendixService;
import com.yizhi.forum.application.service.IPostsService;
import com.yizhi.forum.application.vo.forum.PostsParamsVo;
import com.yizhi.forum.application.vo.forum.PostsStudentVo;
import com.yizhi.forum.application.vo.domain.PostsVo;
import com.yizhi.forum.application.vo.forum.StatisticsForumAccountVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 帖子接口
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@RestController
@RequestMapping("/remote/posts")
public class RemotePostsController implements RemotePostsClient {

    @Autowired
    private IPostsService postsService;
    @Autowired
    private IPostsAppendixService postsAppendixService;

    /**
     * 帖子基本信息修改
     *
     * @param vo
     * @return
     */
    @Override
    @PostMapping("/update/info")
    public Integer updateInfo(@RequestBody PostsVo vo) {

        return postsService.updateInfo(vo);
    }


    @Override
    @PostMapping("/getList")
    public List<PostsVo> getList(@RequestBody PostsParamsVo vo) {

        List<Posts> list = postsService.getList(vo);
        List<PostsVo> lists = new ArrayList<>();
        for (Posts p : list) {
            PostsVo postsVo = new PostsVo();
            BeanUtils.copyProperties(p, postsVo);
            lists.add(postsVo);
        }
        return lists;
    }

    @Override
    @PostMapping("/getMap")
    public Map<Long, PostsStudentVo> getMap(@RequestBody PostsParamsVo vo) {
        return postsService.getMap(vo);
    }

    @Override
    @GetMapping("/get/info")
    public PostsVo getInfo(@RequestParam("postsId") Long postsId) {
        Posts list = postsService.getInfo(postsId);
        PostsVo postsVo = new PostsVo();
        BeanUtils.copyProperties(list, postsVo);
        return postsVo;
    }

    /**
     * 报表需要取得所有已上架的帖子
     *
     * @return
     */
    @Override
    @GetMapping("/getAllPosts")
    public List<PostsVo> getAllPosts() {
        Posts posts = new Posts();
        posts.setStatus(ForumConstans.UP);
        posts.setDeleted(ForumConstans.NOT_DELETED);
        List<Posts> list = postsService.selectList(new EntityWrapper<>(posts));
        List<PostsVo> lists = new ArrayList<>();
        for (Posts p : list) {
            PostsVo postsVo = new PostsVo();
            BeanUtils.copyProperties(p, postsVo);
            lists.add(postsVo);
        }

        return lists;
    }

    /**
     * 报表需要个人的帖子数据Map
     *
     * @return
     */
    @Override
    @GetMapping("/getPersonalDataMap")
    public Map<Long, StatisticsForumAccountVo> getPersonalDataMap() {
        return postsService.getPersonalDataMap();
    }

    /**
     * 首页展示精选中的帖子需要帖子实时名称
     *
     * @param postsIds
     * @return
     */
    @Override
    @GetMapping("/getNames")
    public Map<Long, String> getNames(@RequestParam(value = "postsIds", required = false) List<Long> postsIds) {
        return postsService.getNames(postsIds);
    }

    @Override
    @GetMapping({"/verifyVideo"})
    public boolean verifyVideo(@RequestParam("postsAppendixIds") List<Long> postsAppendixIds) {
        return postsAppendixService.verifyVideo(postsAppendixIds);
    }

    @Override
    @PostMapping({"/updateVideoTranscodeUrl"})
    public boolean updateVideoTranscodeUrl(@RequestBody Map<String, String> var1) {
        return postsAppendixService.updateVideoTranscodeUrl(var1);
    }
}

