package com.yizhi.forum.application.controller.student;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.Posts;
import com.yizhi.forum.application.feign.student.StudentPostsClient;
import com.yizhi.forum.application.service.IPostsService;
import com.yizhi.forum.application.vo.forum.PostsParamsVo;
import com.yizhi.forum.application.vo.forum.PostsSaveVo;
import com.yizhi.forum.application.vo.forum.PostsStudentVo;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 帖子接口
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@RestController
@RequestMapping("/posts/student")
public class StudentPostsController implements StudentPostsClient {

    @Autowired
    IPostsService postsService;
    @Autowired
    private PointRedisFeignClients pointRedisFeignClients;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private IdGenerator idGenerator;


    /**
     * 参数解释见mypage
     *
     * @param type
     * @param keyword
     * @param classifyId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    @GetMapping("/getPageToStudent")
    public Page<PostsStudentVo> getPageToStudent(@RequestParam("type") Integer type,
                                                 @RequestParam(value = "keyword", required = false) String keyword,
                                                 @RequestParam(value = "classifyId", required = false) Long classifyId,
                                                 @RequestParam("pageNo") Integer pageNo,
                                                 @RequestParam("pageSize") Integer pageSize) {
        return postsService.getPageToStudent(type, keyword, classifyId, pageNo, pageSize);
    }

    /**
     * 保存帖子
     *
     * @param vo
     * @return
     */
    @Override
    @PostMapping("/save")
    public Integer studentSave(@RequestBody PostsSaveVo vo) {
        Long id = idGenerator.generate();
        vo.setId(id);
        Integer save = postsService.save(vo);
        if (save > 0 && ForumConstans.UP.equals(vo.getType())) {
//       发积分
            sendPoint(id, ContextHolder.get(), vo.getTitle());
        }
        return save;
    }

    @Override
    @PostMapping("/get")
    public PostsSaveVo studentGet(@RequestBody PostsParamsVo vo) {
        return postsService.get(vo);
    }

    /**
     * 我的发帖 我的收藏 我的管理 分页数据
     *
     * @param type       1：我的发帖 2：我的收藏 3：我的管理
     * @param status     0：草稿  1：下架  2:上架
     * @param keyword    分类名称或帖子标题
     * @param classifyId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    @GetMapping("/myPage")
    public Page<PostsStudentVo> myPage(@RequestParam("type") Integer type,
                                       @RequestParam(value = "status",required = false) Integer status,
                                       @RequestParam(value = "keyword",required = false) String keyword,
                                       @RequestParam(value = "classifyId",required = false) Long classifyId,
                                       @RequestParam("pageNo") Integer pageNo,
                                       @RequestParam("pageSize") Integer pageSize) {
        return postsService.myPage(type, status, keyword, classifyId, pageNo, pageSize);
    }

    /**
     * 学员端修改帖子
     *
     * @param vo
     * @return
     */
    @Override
    @PostMapping("/update")
    public Integer update(@RequestBody PostsSaveVo vo) {
        return postsService.update(vo);
    }

    /**
     * 学员端删除帖子
     *
     * @param vo
     * @return
     */
    @Override
    @PostMapping("/deleted")
    public Integer deleted(@RequestBody PostsParamsVo vo) {
        return postsService.upOrDown(vo);
    }

    @Override
    @PostMapping("/topOrHighlight")
    public Integer topOrHighlight(@RequestBody PostsParamsVo vo) {
        return postsService.topOrHighlight(vo);
    }

    @Override
    @GetMapping("/getMapByIds")
    public Map<Long, PostsStudentVo> getMapByIds(@RequestParam("postsIds") List<Long> postsIds) {
        return postsService.getMapByIds(postsIds);
    }

    /**
     * 判断是否发积分
     *
     * @param postsId 帖子id
     * @param context
     */

    public void sendPoint(Long postsId, RequestContext context, String activityName) {

        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                //论坛回帖发送积分
                PointParamVO pointParamVO = new PointParamVO();
                pointParamVO.setEventName("pointForum");
                pointParamVO.setActivityName(activityName);
                pointParamVO.setActivityType("发布论坛帖子发放积分");
                pointParamVO.setActivitySource("论坛");
                pointParamVO.setCreatePointTime(new Date());
                pointParamVO.setSourceId(postsId);
                pointParamVO.setAccountId(context.getAccountId());
                pointParamVO.setSiteId(context.getSiteId());
                pointParamVO.setCompanyId(context.getCompanyId());
                pointParamVO.setAccountName(context.getAccountName());
                pointParamVO.setOrgId(context.getOrgId());
                pointParamVO.setOperatingPoint(0);
                Posts info = postsService.getInfo(postsId);
                if (info != null) {
                    pointParamVO.setOperatingPoint(info.getSendPoint());
                }
                String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
                // 发送积分
                if (!StringUtils.isEmpty(sitePointId)) {
                    amqpTemplate.convertAndSend("forum", sitePointId);
                }
            }
        });
    }

    /**
     * 获取我的回复（用户的回复收到新的回复未查看时）、我的管理（有回复、举报或帖子未查看）的数量
     *
     * @return
     */

    @Override
    @GetMapping("/getMyNum")
    @ApiOperation(value = "获取我的管理（有回复、举报或帖子未查看）的数量")
    public  Integer getMyNum() {
        return postsService.getMyNum();
    }
}

