package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.comment.application.feign.PdfPagesClient;
import com.yizhi.comment.application.vo.PdfVO;
import com.yizhi.comment.application.vo.domain.PdfPagesVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.forum.application.constans.DocumentType;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsAppendix;
import com.yizhi.forum.application.mapper.PostsAppendixMapper;
import com.yizhi.forum.application.service.IPostsAppendixService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.forum.application.vo.forum.PostsAppendixVo;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.event.AssignmentMediaTranscodingEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsAppendixServiceImpl extends ServiceImpl<PostsAppendixMapper, PostsAppendix> implements IPostsAppendixService {

    private Logger logger = LoggerFactory.getLogger(PostsAppendixServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private PdfPagesClient pdfPagesClient;
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(Long postsId, List<PostsAppendixVo> postsAppendixVos) {
        if (CollectionUtils.isEmpty(postsAppendixVos)) {
            logger.info("附件为空！！！");
            return 0;
        }
        RequestContext context = ContextHolder.get();
        Date now = new Date();
        List<PostsAppendix> postsAppendices = new ArrayList<>(postsAppendixVos.size());
        //转码需要
        List<AssignmentMediaTranscodingEvent.FileDTO> fileDTOS = new ArrayList<>();
        AssignmentMediaTranscodingEvent event = new AssignmentMediaTranscodingEvent();
        event.setType(TpActivityType.TYPE_FORUM);
		List<PdfVO> pdfVOs = new ArrayList<PdfVO>();
        postsAppendixVos.forEach(a -> {
            PostsAppendix appendix = new PostsAppendix();
            BeanUtils.copyProperties(a, appendix);
            appendix.setId(idGenerator.generate());
            appendix.setPostsId(postsId);
            appendix.setCreateById(context.getAccountId());
            appendix.setCreateByName(context.getAccountName());
            appendix.setCreateTime(now);
            appendix.setSiteId(context.getSiteId());
            if (a.getType() != null) {
                if (DocumentType.VIDEO.getType().equals(a.getType()) || DocumentType.AUDIO.getType().equals(a.getType())) {
                    fileDTOS.add(event.new FileDTO(appendix.getId(), a.getUrl()));
                }
            }
            postsAppendices.add(appendix);
            PdfVO pdfVO = new PdfVO();
            pdfVO.setBizId(appendix.getId());
            pdfVO.setBizType(TpActivityType.TYPE_FORUM);
            pdfVO.setSiteId(appendix.getSiteId());
            pdfVO.setCreateById(appendix.getCreateById());
            pdfVO.setOriginalFile(appendix.getName());
            pdfVO.setOriginalFileUrl(appendix.getUrl());
            pdfVOs.add(pdfVO);
        });
        event.setFiles(fileDTOS);
        cloudEventPublisher.publish("assignmentMediaTranscoding", new EventWrapper(0L, event));
		try {
			if (!CollectionUtils.isEmpty(pdfVOs)) {
				logger.info("pdf解析图片发送消息："+pdfVOs.toString());
				cloudEventPublisher.publish("pdfToPng",
						new EventWrapper(0L, pdfVOs));
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.error("pdf解析图片发送消息失败：",e);
		}
        return this.insertBatch(postsAppendices) == true ? 0 : postsAppendices.size();
    }

    @Override
    public List<PostsAppendixVo> getListByPostsId(Long postsId) {
        RequestContext context = ContextHolder.get();

        PostsAppendix postsAppendix = new PostsAppendix();
        postsAppendix.setSiteId(context.getSiteId());
        postsAppendix.setPostsId(postsId);
        EntityWrapper<PostsAppendix> wrapper = new EntityWrapper(postsAppendix);
        List<PostsAppendix> postsAppendices = this.selectList(wrapper);
        if (CollectionUtils.isEmpty(postsAppendices)) {
            return new ArrayList<>();
        }
        List<PostsAppendixVo> postsAppendixVos = new ArrayList<>(postsAppendices.size());

        postsAppendices.forEach(a -> {
            PostsAppendixVo vo = new PostsAppendixVo();
            BeanUtils.copyProperties(a, vo);
            try {
				List<PdfPagesVo> list = pdfPagesClient.list(vo.getId(), TpActivityType.TYPE_FORUM);
                List<com.yizhi.forum.application.vo.forum.PdfPagesVo> pagesVos=new ArrayList<>();

                if (!CollectionUtils.isEmpty(list)) {
                    for (PdfPagesVo p:list) {
                        com.yizhi.forum.application.vo.forum.PdfPagesVo pdfPagesVo=new com.yizhi.forum.application.vo.forum.PdfPagesVo();
                        pdfPagesVo.setId(p.getId());
                        pdfPagesVo.setBizId(p.getBizId());
                        pdfPagesVo.setBizType(p.getBizType());
                        pdfPagesVo.setImageUrl(p.getImageUrl());
                        pdfPagesVo.setPdfPage(p.getPdfPage());
                        pagesVos.add(pdfPagesVo);
                    }
                    vo.setPdfPages(pagesVos);
                }
			} catch (Exception e) {
				// TODO Auto-generated catch block
				logger.error("pdf图片获取失败",e);
			}
            postsAppendixVos.add(vo);
        });
        return postsAppendixVos;
    }

    @Override
    public Integer deletedByPostsId(Long postsId) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();

        PostsAppendix postsAppendix = new PostsAppendix();
        postsAppendix.setDeleted(ForumConstans.DELETED);
        postsAppendix.setUpdateById(context.getAccountId());
        postsAppendix.setUpdateByName(context.getAccountName());
        postsAppendix.setUpdateTime(now);
        EntityWrapper<PostsAppendix> wrapper = new EntityWrapper();
        wrapper.eq("site_id", context.getSiteId()).eq(ForumConstans.FEIlD_POSTS_ID, postsId);

        return this.update(postsAppendix, wrapper) == true ? 1 : 0;
    }

    @Override
    public boolean verifyVideo(List<Long> postsAppendixIds) {
        if (CollectionUtils.isEmpty(postsAppendixIds)) {
            return false;
        }
        PostsAppendix appendix = new PostsAppendix();
        appendix.setIsVideo(1);
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.in(ForumConstans.Field_ID, postsAppendixIds);
        return this.update(appendix, wrapper);
    }

    @Override
    public boolean updateVideoTranscodeUrl(Map<String, String> var1) {
        if (var1 != null) {
            System.out.println("传参：" + var1);

            Long id = Long.valueOf(var1.get("id"));
            String videoSd = var1.get("sd");
            PostsAppendix appendix = new PostsAppendix();
            appendix.setId(id);
            appendix.setAliyunUrl(videoSd);
            if (StringUtils.isEmpty(videoSd)) {
                logger.info("阿里云转码url为空！");
                return false;
            }
            return this.updateById(appendix);
        }
        return false;
    }
}
