package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.Posts;
import com.yizhi.forum.application.domain.PostsClassify;
import com.yizhi.forum.application.domain.PostsClassifyAccount;
import com.yizhi.forum.application.mapper.PostsClassifyMapper;
import com.yizhi.forum.application.service.IPostsClassifyAccountService;
import com.yizhi.forum.application.service.IPostsClassifyService;
import com.yizhi.forum.application.service.IPostsService;
import com.yizhi.forum.application.vo.domain.PostsClassifyVo;
import com.yizhi.forum.application.vo.forum.PostsClassifySettingVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsClassifyServiceImpl extends ServiceImpl<PostsClassifyMapper, PostsClassify> implements IPostsClassifyService {
    private Logger logger = LoggerFactory.getLogger(PostsClassifyServiceImpl.class);
    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IPostsService postsService;
    @Autowired
    private IPostsClassifyAccountService postsClassifyAccountService;


    @Override
    public Boolean save(PostsClassifyVo postsClassify) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();
        Long id = idGenerator.generate();

        postsClassify.setId(id);
        postsClassify.setDeleted(ForumConstans.NOT_DELETED);
        postsClassify.setSort(ForumConstans.ZERO);
        postsClassify.setCompanyId(context.getCompanyId());
        postsClassify.setSiteId(context.getSiteId());
        postsClassify.setOrgId(context.getOrgId());
        postsClassify.setCreateById(context.getAccountId());
        postsClassify.setCreateByName(context.getAccountName());
        postsClassify.setCreateTime(now);
        PostsClassify pc=new PostsClassify();
        BeanUtils.copyProperties(postsClassify,pc);
        if (this.insert(pc)) {
            Integer maxSort = this.baseMapper.getMaxSort(context.getSiteId());
            pc = new PostsClassify();
            pc.setId(id);
            pc.setSort(maxSort + 1);
            this.baseMapper.updateById(pc);
            return true;
        }
        return false;
    }

    @Override
    public Integer move(Integer type, Long id) {
        RequestContext context = ContextHolder.get();

        PostsClassify postsClassify = this.selectById(id);
        Integer sourceSort = postsClassify.getSort();
        PostsClassify targetClassify = this.baseMapper.getSort(id, context.getSiteId(), type);
        if (targetClassify != null) {
            Integer targetSort = targetClassify.getSort();
            if (null != targetSort) {
                postsClassify.setSort(targetSort);
                this.updateById(postsClassify);
                targetClassify.setSort(sourceSort);
                this.updateById(targetClassify);
            }
            return 1;
        } else {
            return type == 1 ? -1 : -2;//-1到顶了   -2到底了
        }
    }

    @Override
    public Integer delete(Long id) {
        RequestContext context = ContextHolder.get();

        Posts posts = new Posts();
        posts.setPostsClassifyId(id);
        posts.setSiteId(context.getSiteId());
        EntityWrapper wrapper = new EntityWrapper(posts);
        int count = postsService.selectCount(wrapper);
        if (count > 0) {
            return -1;//该分类下有帖子
        }
        PostsClassify postsClassify = this.baseMapper.selectById(id);
        postsClassify.setDeleted(ForumConstans.DELETED);
        postsClassify.setUpdateById(context.getAccountId());
        postsClassify.setUpdateByName(context.getAccountName());
        postsClassify.setUpdateTime(new Date());
        if (this.updateById(postsClassify)) {
            return 1;
        }
        return 0;
    }

    @Override
    public Boolean updateSetting(PostsClassifySettingVo vo) {
        RequestContext context = ContextHolder.get();

        PostsClassify postsClassify = new PostsClassify();
        BeanUtils.copyProperties(vo, postsClassify);
        postsClassify.setUpdateById(context.getAccountId());
        postsClassify.setUpdateByName(context.getAccountName());
        postsClassify.setUpdateTime(new Date());
        this.updateById(postsClassify);
        postsClassifyAccountService.save(vo);
        return true;
    }

    @Override
    public PostsClassifySettingVo getSetting(Long id) {
        PostsClassify postsClassify = this.selectById(id);
        PostsClassifySettingVo vo = null;
        if (postsClassify != null) {
            vo = new PostsClassifySettingVo();
            BeanUtils.copyProperties(postsClassify, vo);
            vo.setPostsClassifyAccountVos(postsClassifyAccountService.getListByClassifyId(id));
        }
        return vo;
    }

    @Override
    public Page<PostsClassifyVo> getPage(String name, Integer pageNo, Integer pageSize) {
        Page page = new Page(pageNo, pageSize);

        PostsClassify postsClassify = new PostsClassify();
        postsClassify.setSiteId(ContextHolder.get().getSiteId());
        EntityWrapper<PostsClassify> wrapper = new EntityWrapper<>(postsClassify);
        wrapper.eq("deleted",0);
        if (!StringUtils.isEmpty(name)) {
            wrapper.like(ForumConstans.FEIlD_NAME, name);
        }
        wrapper.orderBy(ForumConstans.Field_SORT,false).orderBy("create_time",false);
        page = this.selectPage(page, wrapper);
        List<PostsClassify> postsClassifies = page.getRecords();

        if (!CollectionUtils.isEmpty(postsClassifies)) {
            List<PostsClassifyVo> classifyVos = new ArrayList<>(postsClassifies.size());
            postsClassifies.forEach(a -> {
                PostsClassifyVo vo = new PostsClassifyVo();
                BeanUtils.copyProperties(a, vo);
                classifyVos.add(vo);
            });
            page.setRecords(classifyVos);
        }
        return page;
    }

    @Override
    public Page<PostsClassifyVo> getPageToStudent(String name, Boolean canSend, Integer pageNo, Integer pageSize) {
        Page page = new Page(pageNo, pageSize);

        PostsClassify postsClassify = new PostsClassify();
        postsClassify.setSiteId(ContextHolder.get().getSiteId());
        EntityWrapper<PostsClassify> wrapper = new EntityWrapper<>(postsClassify);
        if (!StringUtils.isEmpty(name)) {
            wrapper.like(ForumConstans.FEIlD_NAME, name);
        }
        if (canSend) {
            //获取能发帖的用户范围
            Set<Long> classifyIds =  new HashSet<>();;
            List<PostsClassifyAccount> classifyAccounts = postsClassifyAccountService.getListByAccountId();
            if (!CollectionUtils.isEmpty(classifyAccounts)) {
              for (PostsClassifyAccount a:classifyAccounts){
                  classifyIds.add(a.getPostsClassifyId());
              }
            }
            wrapper.andNew().eq(ForumConstans.Field_SEND_SETTING, ForumConstans.SITE_ACCOUNT);
            if (!CollectionUtils.isEmpty(classifyIds)) {
                wrapper.or().in(ForumConstans.Field_ID, classifyIds);
            }
        }
        wrapper.orderBy(ForumConstans.Field_SORT);
        page = this.selectPage(page, wrapper);
        List<PostsClassify> postsClassifies = page.getRecords();
        if (!CollectionUtils.isEmpty(postsClassifies)) {
            List<PostsClassifyVo> classifyVos = new ArrayList<>(postsClassifies.size());
            postsClassifies.forEach(a -> {
                PostsClassifyVo vo = new PostsClassifyVo();
                BeanUtils.copyProperties(a, vo);
                classifyVos.add(vo);
            });
            page.setRecords(classifyVos);
        }
        return page;
    }

    @Override
    public Map<Long, PostsClassifyVo> getClassifyMap(RequestContext context) {
        Map<Long, PostsClassifyVo> map = null;
        PostsClassify postsClassify = new PostsClassify();
        postsClassify.setSiteId(context.getSiteId());
        EntityWrapper<PostsClassify> wrapper = new EntityWrapper<>(postsClassify);
        List<PostsClassify> postsClassifies = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(postsClassifies)) {
            map = new HashMap<>(postsClassifies.size());
            for (PostsClassify a : postsClassifies) {
                if (!map.containsKey(a.getId())){
                    PostsClassifyVo pc=new PostsClassifyVo();
                    BeanUtils.copyProperties(a,pc);
                    map.put(a.getId(),pc);
                }
            }
        }
        return map;
    }

    @Override
    public Map<Long, String> getClassifyNameMap(List<Long> ids) {
        Map<Long, String> map = null;
        PostsClassify postsClassify = new PostsClassify();
        postsClassify.setSiteId(ContextHolder.get().getSiteId());
        EntityWrapper<PostsClassify> wrapper = new EntityWrapper<>(postsClassify);
        wrapper.in("id",ids);
        List<PostsClassify> postsClassifies = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(postsClassifies)) {
            map = new HashMap<>(postsClassifies.size());
            for (PostsClassify a : postsClassifies) {
                if (!map.containsKey(a.getId())){
                    map.put(a.getId(),a.getName());
                }
            }
        }
        return map;
    }
}
