package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsFavorite;
import com.yizhi.forum.application.mapper.PostsFavoriteMapper;
import com.yizhi.forum.application.service.IPostsFavoriteService;
import com.yizhi.forum.application.service.IPostsService;
import com.yizhi.forum.application.vo.domain.PostsFavoriteVo;
import com.yizhi.forum.application.vo.domain.PostsVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsFavoriteServiceImpl extends ServiceImpl<PostsFavoriteMapper, PostsFavorite> implements IPostsFavoriteService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IPostsService postsService;

    @Override
    public List<Long> getPostsIds(Long accountId, Long siteId) {

        PostsFavorite postsFavorite = new PostsFavorite();
        postsFavorite.setSiteId(siteId);
        //1：帖子
        postsFavorite.setAccountId(accountId);
        EntityWrapper wrapper = new EntityWrapper(postsFavorite);
        List<PostsFavorite> favorites = this.baseMapper.selectList(wrapper);
        if (!CollectionUtils.isEmpty(favorites)) {
            return favorites.stream().map(a -> a.getPostsId()).collect(Collectors.toList());
        }
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(PostsFavoriteVo postsFavorite) {

        RequestContext context = ContextHolder.get();
        Date date = new Date();
        postsFavorite.setId(idGenerator.generate());
        postsFavorite.setSiteId(context.getSiteId());
        postsFavorite.setAccountId(context.getAccountId());
        postsFavorite.setCompanyId(context.getCompanyId());
        postsFavorite.setOrgId(context.getOrgId());
        postsFavorite.setCreateById(context.getAccountId());
        postsFavorite.setCreateByName(context.getAccountName());
        postsFavorite.setCreateTime(date);
        PostsFavorite postsFavorite1=new PostsFavorite();
        BeanUtils.copyProperties(postsFavorite,postsFavorite1);
        if (this.insert(postsFavorite1)){
            PostsVo vo = new PostsVo();
            vo.setId(postsFavorite.getPostsId());
            vo.setFavoriteTime(date);
            postsService.updateInfo(vo);
            return 1;
        }
        return 0;
    }

    @Override
    public Integer deleted(PostsFavoriteVo postsFavorite) {
        RequestContext context = ContextHolder.get();
        PostsFavorite update = new PostsFavorite();
        update.setDeleted(ForumConstans.DELETED);
        update.setUpdateById(context.getAccountId());
        update.setUpdateByName(context.getAccountName());
        update.setUpdateTime(new Date());
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq(ForumConstans.FEIlD_POSTS_ID, postsFavorite.getPostsId());
        return this.update(update, wrapper) == true ? 1 : 0;
    }

    @Override
    public Integer hasFavoriteByPostsId(Long postsId,Boolean accountId) {
        RequestContext context = ContextHolder.get();
        PostsFavorite postsFavorite = new PostsFavorite();
        postsFavorite.setSiteId(context.getSiteId());
        postsFavorite.setPostsId(postsId);
        if (accountId){
            postsFavorite.setAccountId(context.getAccountId());
        }
        EntityWrapper wrapper = new EntityWrapper(postsFavorite);
        return this.selectCount(wrapper);
    }

    @Override
    public Map<Long, Integer> hasFavoriteByPostsIds(List<Long> postsIds) {
        Map<Long, Integer> map = null;
        PostsFavorite postsFavorite = new PostsFavorite();
        postsFavorite.setSiteId(ContextHolder.get().getSiteId());
        EntityWrapper wrapper = new EntityWrapper(postsFavorite);
        wrapper.in(ForumConstans.FEIlD_POSTS_ID, postsIds);
        List<PostsFavorite> postsFavorites = this.selectList(wrapper);
        if (CollectionUtils.isEmpty(postsFavorites)) {
            map = new HashMap<>(postsFavorites.size());
            for (PostsFavorite a : postsFavorites) {
                if (!map.containsKey(a.getPostsId())) {
                    map.put(a.getPostsId(), 1);
                } else {
                    Integer num = map.get(a.getPostsId());
                    map.put(a.getPostsId(), num + 1);
                }
            }
        }
        return map;
    }
}
