package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsRelation;
import com.yizhi.forum.application.mapper.PostsRelationMapper;
import com.yizhi.forum.application.service.IPostsRelationService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.forum.application.vo.forum.PostsRelationVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsRelationServiceImpl extends ServiceImpl<PostsRelationMapper, PostsRelation> implements IPostsRelationService {
    private Logger logger = LoggerFactory.getLogger(PostsAppendixServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(Long postsId, List<PostsRelationVo> postsRelationVos) {
        if (CollectionUtils.isEmpty(postsRelationVos)) {
            logger.info("关联活动为空！！！");
            return 0;
        }
        RequestContext context = ContextHolder.get();
        Date now = new Date();
        List<PostsRelation> postsRelations = new ArrayList<>(postsRelationVos.size());

        postsRelationVos.forEach(a -> {
            PostsRelation postsRelation = new PostsRelation();
            BeanUtils.copyProperties(a, postsRelation);
            postsRelation.setId(idGenerator.generate());
            postsRelation.setPostsId(postsId);
            postsRelation.setCreateById(context.getAccountId());
            postsRelation.setCreateByName(context.getAccountName());
            postsRelation.setCreateTime(now);
            postsRelations.add(postsRelation);
        });
        return this.insertBatch(postsRelations) == true ? 0 : postsRelations.size();
    }

    @Override
    public List<PostsRelationVo> getListByPostsId(Long postsId) {

        PostsRelation postsRelation = new PostsRelation();
        postsRelation.setPostsId(postsId);
        EntityWrapper<PostsRelation> wrapper = new EntityWrapper(postsRelation);
        List<PostsRelation> postsRelations = this.selectList(wrapper);
        if (CollectionUtils.isEmpty(postsRelations)) {
            return new ArrayList<>();
        }
        List<PostsRelationVo> postsRelationVos = new ArrayList<>(postsRelations.size());
        postsRelations.forEach(a -> {
            PostsRelationVo vo = new PostsRelationVo();
            BeanUtils.copyProperties(a, vo);
            postsRelationVos.add(vo);
        });
        return postsRelationVos;
    }

    @Override
    public Integer deletedByPostsId(Long postsId) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();

        PostsRelation postsRelation = new PostsRelation();
        postsRelation.setDeleted(ForumConstans.DELETED);
        postsRelation.setUpdateById(context.getAccountId());
        postsRelation.setUpdateByName(context.getAccountName());
        postsRelation.setUpdateTime(now);
        EntityWrapper<PostsRelation> wrapper = new EntityWrapper();
        wrapper.eq(ForumConstans.FEIlD_POSTS_ID, postsId);
        return this.update(postsRelation, wrapper) == true ? 1 : 0;
    }
}
