package com.yizhi.forum.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.forum.application.constans.ForumConstans;
import com.yizhi.forum.application.domain.PostsVisibleRange;
import com.yizhi.forum.application.mapper.PostsVisibleRangeMapper;
import com.yizhi.forum.application.service.IPostsVisibleRangeService;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.forum.application.vo.forum.PostsVisibleRangeVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-09
 */
@Service
public class PostsVisibleRangeServiceImpl extends ServiceImpl<PostsVisibleRangeMapper, PostsVisibleRange> implements IPostsVisibleRangeService {
    private Logger logger = LoggerFactory.getLogger(PostsVisibleRangeServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(Long postsId, List<PostsVisibleRangeVo> postsVisibleRangeVos) {
        if (CollectionUtils.isEmpty(postsVisibleRangeVos)) {
            logger.info("指定用户为空！！！");
            return 0;
        }
        RequestContext context = ContextHolder.get();
        Date now = new Date();
        List<PostsVisibleRange> visibleRanges = new ArrayList<>(postsVisibleRangeVos.size());

        postsVisibleRangeVos.forEach(a -> {
            PostsVisibleRange visibleRange = new PostsVisibleRange();
            BeanUtils.copyProperties(a, visibleRange);
            visibleRange.setId(idGenerator.generate());
            visibleRange.setPostsId(postsId);
            visibleRange.setCreateById(context.getAccountId());
            visibleRange.setCreateByName(context.getAccountName());
            visibleRange.setCreateTime(now);
            visibleRange.setSiteId(context.getSiteId());
            visibleRanges.add(visibleRange);
        });
        return this.insertBatch(visibleRanges) == true ? 0 : visibleRanges.size();
    }

    @Override
    public List<PostsVisibleRangeVo> getListByPostsId(Long postsId) {

        RequestContext context = ContextHolder.get();

        PostsVisibleRange postsVisibleRange = new PostsVisibleRange();
        postsVisibleRange.setDeleted(ForumConstans.NOT_DELETED);
        postsVisibleRange.setPostsId(postsId);
        postsVisibleRange.setSiteId(context.getSiteId());
        EntityWrapper<PostsVisibleRange> wrapper = new EntityWrapper(postsVisibleRange);
        List<PostsVisibleRange> postsVisibleRanges = this.selectList(wrapper);
        if (CollectionUtils.isEmpty(postsVisibleRanges)) {
            return new ArrayList<>();
        }
        List<PostsVisibleRangeVo> postsVisibleRangeVos = new ArrayList<>(postsVisibleRanges.size());

        postsVisibleRanges.forEach(a -> {
            PostsVisibleRangeVo vo = new PostsVisibleRangeVo();
            BeanUtils.copyProperties(a, vo);
            postsVisibleRangeVos.add(vo);
        });
        return postsVisibleRangeVos;
    }

    @Override
    public Integer deletedByPostsId(Long postsId) {
        RequestContext context = ContextHolder.get();
        Date now = new Date();

        PostsVisibleRange postsVisibleRange = new PostsVisibleRange();
        postsVisibleRange.setDeleted(1);
        postsVisibleRange.setUpdateById(context.getAccountId());
        postsVisibleRange.setUpdateByName(context.getAccountName());
        postsVisibleRange.setUpdateTime(now);
        EntityWrapper<PostsVisibleRange> wrapper = new EntityWrapper();
        wrapper.eq("site_id",context.getSiteId())
                .eq(ForumConstans.FEIlD_POSTS_ID, postsId);
        return this.update(postsVisibleRange, wrapper) == true ? 1 : 0;
    }

    @Override
    public List<Long> getPostsIds(RequestContext context) {
        List<Long> postsIds = null;
        PostsVisibleRange postsVisibleRange = new PostsVisibleRange();
        postsVisibleRange.setSiteId(context.getSiteId());
        EntityWrapper<PostsVisibleRange> wrapper = new EntityWrapper(postsVisibleRange);
        wrapper.in(ForumConstans.Field_ACCOUNT_ID, context.getRelationIds());
        List<PostsVisibleRange> postsVisibleRanges = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(postsVisibleRanges)){
            postsIds = new ArrayList<>(postsVisibleRanges.size());
            postsIds = postsVisibleRanges.parallelStream().map(a->{return a.getPostsId();}).collect(Collectors.toList());
        }
        return postsIds;
    }


}
